/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2012 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.graphicsGL.ui.light;

import static org.fortiss.tooling.graphicsGL.ui.util.FloatBufferUtils.toFloatBuffer;
import static org.fortiss.tooling.graphicsGL.util.Graphics3DModelElementFactory.color;
import static org.lwjgl.opengl.GL11.GL_LIGHT0;
import static org.lwjgl.opengl.GL11.glDisable;

import java.nio.FloatBuffer;

import org.fortiss.tooling.graphicsGL.model.Color3D;
import org.fortiss.tooling.graphicsGL.ui.util.GLPrimitives;

/**
 * Class representing a light source. It is initialized with a GL light number (GL_LIGHT0, ...,
 * GL_LIGHT7) and a {@link Color3D}, which determines the light color for ambient and diffuse.
 * Specular light is set to white. The default position is (10,10,10).
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 0CDD15394CBA85A529E6042BE9D98DB3
 */
public class Light {

	/** The GL_LIGHTx to be used. */
	private final int glLightNumber;

	/** Constructor. */
	public Light(int glLight, Color3D color) {
		this(glLight, color, color, color);
	}

	/** Constructor. */
	public Light(int glLight, Color3D ambient, Color3D diffuse, Color3D specular) {
		glLightNumber = glLight;
		setLightColor(ambient, diffuse, specular);
		setPosition(10.0f, 10.0f, 10.0f);
	}

	/** Constructor using GL_LIGHT0 and white light color. */
	public Light() {
		this(GL_LIGHT0, color(1.0f, 1.0f, 1.0f));
	}

	/** Constructor using GL_LIGHT0 and the given light color. */
	public Light(Color3D color) {
		this(GL_LIGHT0, color);
	}

	/** Sets the light color. */
	public final void setLightColor(Color3D ambient, Color3D diffuse, Color3D specular) {
		ambientColor = toFloatBuffer(ambient.getRed(), ambient.getGreen(), ambient.getBlue(), 1.0f);
		diffuseColor = toFloatBuffer(diffuse.getRed(), diffuse.getGreen(), diffuse.getBlue(), 1.0f);
		specularColor =
				toFloatBuffer(specular.getRed(), specular.getGreen(), specular.getBlue(), 1.0f);
	}

	/** The position. */
	private FloatBuffer lightPosition;
	/** The ambient color. */
	private FloatBuffer ambientColor;
	/** The diffuse color. */
	private FloatBuffer diffuseColor;
	/** The specular color. */
	private FloatBuffer specularColor;

	/** Enables the light in GL. */
	public void enableLight() {
		GLPrimitives.enableLight(glLightNumber, lightPosition, ambientColor, diffuseColor,
				specularColor);
	}

	/** Disables the light in GL. */
	public void disableLight() {
		glDisable(glLightNumber);
	}

	/** Sets the light position. */
	public void setPosition(float x, float y, float z) {
		lightPosition = toFloatBuffer(new float[] {x, y, z, 1.0f});
	}
}
